@@ -85,9 +85,14 @@ module RBS
         case node.type
         when :CLASS
           class_name, super_class, *class_body = node.children
+          super_class_name = const_to_name(super_class)
+          if super_class_name.nil?
+            # Give up detect super class e.g. `class Foo < Struct.new(:bar)`
+            super_class = nil
+          end
           kls = AST::Declarations::Class.new(
             name: const_to_name(class_name),
-            super_class: super_class && AST::Declarations::Class::Super.new(name: const_to_name(super_class), args: [], location: nil),
+            super_class: super_class && AST::Declarations::Class::Super.new(name: super_class_name, args: [], location: nil),
             type_params: [],
             members: [],
             annotations: [],
@@ -427,9 +432,9 @@ module RBS
 
           case value
           when nil, :NODE_SPECIAL_REQUIRED_KEYWORD
-            fun.required_keywords[name] = Types::Function::Param.new(name: name, type: untyped)
+            fun.required_keywords[name] = Types::Function::Param.new(name: nil, type: untyped)
           when RubyVM::AbstractSyntaxTree::Node
-            fun.optional_keywords[name] = Types::Function::Param.new(name: name, type: param_type(value))
+            fun.optional_keywords[name] = Types::Function::Param.new(name: nil, type: param_type(value))
           else
             raise "Unexpected keyword arg value: #{value}"
           end
@@ -444,28 +449,48 @@ module RBS
 
       def function_return_type_from_body(node)
         body = node.children[2]
-        return Types::Bases::Nil.new(location: nil) unless body
+        body_type(body)
+      end
 
-        if body.type == :BLOCK
-          return_stmts = any_node?(body) do |n|
-            n.type == :RETURN
-          end&.map do |return_node|
-            returned_value = return_node.children[0]
-            returned_value ? literal_to_type(returned_value) : Types::Bases::Nil.new(location: nil)
-          end || []
-          last_node = body.children.last
-          last_evaluated =  last_node ? literal_to_type(last_node) : Types::Bases::Nil.new(location: nil)
-          types_to_union_type([*return_stmts, last_evaluated])
+      def body_type(node)
+        return Types::Bases::Nil.new(location: nil) unless node
+
+        case node.type
+        when :IF, :UNLESS
+          if_unless_type(node)
+        when :BLOCK
+          block_type(node)
         else
-          literal_to_type(body)
+          literal_to_type(node)
         end
       end
 
+      def if_unless_type(node)
+        raise unless node.type == :IF || node.type == :UNLESS
+
+        _exp_node, true_node, false_node = node.children
+        types_to_union_type([body_type(true_node), body_type(false_node)])
+      end
+
+      def block_type(node)
+        raise unless node.type == :BLOCK
+
+        return_stmts = any_node?(node) do |n|
+          n.type == :RETURN
+        end&.map do |return_node|
+          returned_value = return_node.children[0]
+          returned_value ? literal_to_type(returned_value) : Types::Bases::Nil.new(location: nil)
+        end || []
+        last_node = node.children.last
+        last_evaluated =  last_node ? literal_to_type(last_node) : Types::Bases::Nil.new(location: nil)
+        types_to_union_type([*return_stmts, last_evaluated])
+      end
+
       def literal_to_type(node)
         case node.type
         when :STR
           lit = node.children[0]
-          if lit.match?(/\A[ -~]+\z/)
+          if lit.ascii_only?
             Types::Literal.new(literal: lit, location: nil)
           else
             BuiltinNames::String.instance_type
@@ -486,7 +511,7 @@ module RBS
           lit = node.children[0]
           case lit
           when Symbol
-            if lit.match?(/\A[ -~]+\z/)
+            if lit.to_s.ascii_only?
               Types::Literal.new(literal: lit, location: nil)
             else
               BuiltinNames::Symbol.instance_type
@@ -498,15 +523,15 @@ module RBS
             Types::ClassInstance.new(name: type_name, args: [], location: nil)
           end
         when :ZLIST, :ZARRAY
-          BuiltinNames::Array.instance_type([untyped])
+          BuiltinNames::Array.instance_type(untyped)
         when :LIST, :ARRAY
           elem_types = node.children.compact.map { |e| literal_to_type(e) }
           t = types_to_union_type(elem_types)
-          BuiltinNames::Array.instance_type([t])
+          BuiltinNames::Array.instance_type(t)
         when :DOT2, :DOT3
           types = node.children.map { |c| literal_to_type(c) }
           type = range_element_type(types)
-          BuiltinNames::Range.instance_type([type])
+          BuiltinNames::Range.instance_type(type)
         when :HASH
           list = node.children[0]
           if list
@@ -534,7 +559,7 @@ module RBS
           else
             key_type = types_to_union_type(key_types)
             value_type = types_to_union_type(value_types)
-            BuiltinNames::Hash.instance_type([key_type, value_type])
+            BuiltinNames::Hash.instance_type(key_type, value_type)
           end
         when :CALL
           receiver, method_name, * = node.children
