@@ -16,6 +16,9 @@ class World < Hello
 end
 
 module Foo
+end
+
+class Bar < Struct.new(:bar)
 end
     EOR
 
@@ -29,6 +32,9 @@ class World < Hello
 end
 
 module Foo
+end
+
+class Bar
 end
     EOF
   end
@@ -56,11 +62,11 @@ end
 
     assert_write parser.decls, <<-EOF
 class Hello
-  def hello: (untyped a, ?::Integer b, *untyped c, untyped d, e: untyped e, ?f: ::Integer f, **untyped g) { () -> untyped } -> nil
+  def hello: (untyped a, ?::Integer b, *untyped c, untyped d, e: untyped, ?f: ::Integer, **untyped g) { () -> untyped } -> nil
 
   def self.world: () { (untyped, untyped, untyped, x: untyped, y: untyped) -> untyped } -> untyped
 
-  def kw_req: (a: untyped a) -> nil
+  def kw_req: (a: untyped) -> nil
 end
     EOF
   end
@@ -72,7 +78,7 @@ end
 class Hello
   def initialize() 'foo' end
 
-  def str() "foo\nbar" end
+  def str() "こんにちは" end
   def str_lit() "foo" end
   def dstr() "f#{x}oo" end
   def xstr() `ls` end
@@ -223,6 +229,85 @@ end
     EOF
   end
 
+  def test_defs_return_type_with_if
+    parser = RB.new
+
+    rb = <<-EOR
+class ReturnTypeWithIF
+  def with_if
+    if foo?
+      true
+    end
+  end
+
+  def with_if_and_block
+    if foo?
+      foo
+      return false if bar?
+      true
+    end
+  end
+
+  def with_else_and_bool
+    if foo?
+      true
+    else
+      false
+    end
+  end
+
+  def with_else_and_elsif_and_bool_nil
+    if foo?
+    elsif bar?
+      true
+    else
+      false
+    end
+  end
+
+  def with_nested_if
+    if foo?
+      if bar?
+        if baz?
+          1
+        else
+          2
+        end
+      else
+        3
+      end
+    else
+      4
+    end
+  end
+
+  def with_unless
+    unless foo?
+      :sym
+    end
+  end
+end
+EOR
+
+    parser.parse(rb)
+
+    assert_write parser.decls, <<-EOR
+class ReturnTypeWithIF
+  def with_if: () -> (true | nil)
+
+  def with_if_and_block: () -> (false | true | nil)
+
+  def with_else_and_bool: () -> (true | false)
+
+  def with_else_and_elsif_and_bool_nil: () -> (nil | true | false)
+
+  def with_nested_if: () -> (1 | 2 | 3 | 4)
+
+  def with_unless: () -> (:sym | nil)
+end
+EOR
+  end
+
   def test_sclass
     parser = RB.new
 
@@ -636,6 +721,12 @@ H: { id: 123 }
     EOF
   end
 
+  def test_invalid_byte_sequence_in_utf8
+    parser = RB.new
+    parser.parse('A = "\xff"')
+    assert_write parser.decls, "A: ::String\n"
+  end
+
   def test_argumentless_fcall
     parser = RB.new
 
@@ -780,6 +871,23 @@ end
     RBS
   end
 
+  def test_literal_to_type
+    parser = RBS::Prototype::RB.new
+    [
+      [%{"abc"}, %{"abc"}],
+      [%{:abc}, %{:abc}],
+      [%{[]}, %{::Array[untyped]}],
+      [%{[true]}, %{::Array[true]}],
+      [%{1..2}, %{::Range[::Integer]}],
+      [%{{}}, %{::Hash[untyped, untyped]}],
+      [%{{a: nil}}, %{ { a: nil } }],
+      [%{{"a" => /b/}}, %{ ::Hash[::String, ::Regexp] }],
+    ].each do |rb, rbs|
+      node = RubyVM::AbstractSyntaxTree.parse(rb).children[2]
+      assert_equal RBS::Parser.parse_type(rbs), parser.literal_to_type(node)
+    end
+  end
+
   if RUBY_VERSION >= '2.7'
     def test_argument_forwarding
       parser = RB.new
